from allauth.socialaccount import providers
from allauth.socialaccount.providers.base import ProviderAccount
from allauth.socialaccount.providers.oauth2.provider import OAuth2Provider


class D120Account(ProviderAccount):
    pass


class D120Provider(OAuth2Provider):
    id = 'd120'
    name = 'D120 OAuth2 Provider'
    account_class = D120Account

    def extract_uid(self, data):
        return str(data['username'])

    def extract_common_fields(self, data):
        return dict(username=data['username'],
                    email=data.get('email', ''),
                    first_name=data.get('firstName', ''),
                    last_name=data.get('lastName', ''), )

    def get_default_scope(self):
        scope = ['microprofile-jwt']
        return scope


providers.registry.register(D120Provider)



from django.contrib.auth.models import Group
from django.db import models
from django.utils.translation import gettext as _


class GroupSync(models.Model):
    external_group = models.CharField(max_length=100, verbose_name=_("External group name"))
    django_group = models.ForeignKey(Group, verbose_name=_("Django Group"), on_delete=models.CASCADE)

    class Meta:
        verbose_name = _("Group synchronization")
        verbose_name_plural = _("Group synchronizations")
        unique_together = ['external_group', 'django_group']

    def __str__(self):
        return "{} -> {}".format(self.external_group, self.django_group)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class D120ProviderConfig(AppConfig):
    name = 'allauth_d120_provider'
    verbose_name = _("D120 Authentication Provider")

    def ready(self):
        # noinspection PyUnresolvedReferences
        from .signals import sig_sync_user_to_groups



# Generated by Django 3.0.1 on 2019-12-25 20:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='GroupSync',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('external_group', models.CharField(max_length=100, verbose_name='External group name')),
                ('django_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='auth.Group', verbose_name='Django Group')),
            ],
            options={
                'verbose_name': 'Group synchronization',
                'verbose_name_plural': 'Group synchronizations',
                'unique_together': {('external_group', 'django_group')},
            },
        ),
    ]



from allauth.socialaccount.models import SocialAccount
from django.contrib.auth.signals import user_logged_in
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.dispatch import receiver

from .models import GroupSync
from .provider import D120Provider


@receiver(user_logged_in, dispatch_uid="6760a6e0-8d7a-466e-bdea-91bdd70da93a")
def sig_sync_user_to_groups(sender, user, request, **kwargs):
    try:
        social_account = SocialAccount.objects.get(user=user, provider=D120Provider.id)
    except ObjectDoesNotExist:
        pass
    else:
        external_groups = social_account.extra_data.get('groups', list())
        sync_groups = GroupSync.objects
        groups_to_remove = sync_groups.filter(~Q(external_group__in=external_groups))
        groups_to_add = sync_groups.filter(external_group__in=external_groups)

        for group in groups_to_remove:
            user.groups.remove(group.django_group)

        for group in groups_to_add:
            user.groups.add(group.django_group)

        user.save()



from allauth.account.adapter import DefaultAccountAdapter
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


class DisableSignUpAdapter(DefaultAccountAdapter):
    def is_open_for_signup(self, request):
        return False


class SocialAccountAdapter(DefaultSocialAccountAdapter):
    def is_open_for_signup(self, request, sociallogin):
        return True

    def validate_disconnect(self, account, accounts):
        raise ValidationError(_("Disconnecting the account is not allowed."))



from setuptools import find_packages, setup


with open('README.md') as fh:
    long_description = fh.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='django-allauth-d120',
    version='0.0.5',
    packages=find_packages(),
    include_package_data=True,
    license='AGPL',
    description='Allauth provider for the D120 SSO',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/d120/django-allauth-d120',
    author='ckleemann',
    author_email='ckleemann@d120.de',
    install_requires=requirements,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)



from functools import partial

import requests
from allauth.socialaccount import app_settings
from allauth.socialaccount.providers.oauth2.views import (OAuth2Adapter, OAuth2LoginView, OAuth2CallbackView)

from .provider import D120Provider


class D120Adapter(OAuth2Adapter):
    provider_id = D120Provider.id

    provider_default_url = 'https://sso.fachschaft.informatik.tu-darmstadt.de'
    provider_default_realm = 'master'

    settings = app_settings.PROVIDERS.get(provider_id, {})
    provider_base_url = settings.get('URL', provider_default_url)
    provider_realm = settings.get('realm', provider_default_realm)
    provider_base_string = '{base}/auth/realms/{realm}/protocol/openid-connect/{endpoint}/'
    provider_base = partial(provider_base_string.format, base=provider_base_url, realm=provider_realm)

    # Fetched programmatically, must be reachable from container
    access_token_url = provider_base(endpoint='token')
    profile_url = provider_base(endpoint='userinfo')

    # Accessed by the user browser, must be reachable by the host
    authorize_url = provider_base(endpoint='auth')

    def complete_login(self, request, app, token, **kwargs):
        headers = {'Authorization': 'Bearer {0}'.format(token.token)}
        resp = requests.get(self.profile_url, headers=headers)
        extra_data = resp.json()
        return self.get_provider().sociallogin_from_response(request, extra_data)


oauth2_login = OAuth2LoginView.adapter_view(D120Adapter)
oauth2_callback = OAuth2CallbackView.adapter_view(D120Adapter)
